/**
	Achim Ioan Alexandru
	333CA
	
	Proiectarea cu Microprocesoare
	
	Proiect: Robot foto-senzitiv
**/

#include <avr\io.h>
#include <avr\interrupt.h>

#define F_CPU 16000000

#include <util/delay.h>

#include <stdlib.h>
#include <stdio.h>

#define MEAN(x,y) (((x)+(y))/2)

#define BDIR ((1 << PB2) | (1 << PB3) | (1 << PB4))
#define ADIR 0x00

#define LIMIT 50

#define LOOP_ALL for(;;);

#define SET_B_BIT(x)  (PORTB |= (1 << x))
#define RESET_B_BIT(x) (PORTB &= ~(1 << x))

#define _TIMER_OVERFLOW_TIME 7812
#define _LIGHT_INTENSITY_LIMIT 50
#define _START_POINT_OF_ENUM 0

typedef enum
{
	CHANNEL_NONE_ACTIVE = _START_POINT_OF_ENUM,
	CHANNEL_ACTIVE_LEFT_EYE,
	CHANNEL_ACTIVE_RIGHT_EYE
}_Channel;

typedef enum
{
	VALUE_IN_LIMIT = _START_POINT_OF_ENUM,
	VALUE_TOWARDS_LEFT,
	VALUE_TOWARDS_RIGHT
}_Limit;

int _GetADCValue(_Channel ch)
{
	ADMUX = (1 << REFS0) | (ch);
	
	ADCSRA |= (1 << ADSC);
	
	for(;(ADCSRA & 0x10) == 0;);
	ADCSRA |= 0x10;
	
	return ADCW;
}

void _InitADCBehavior(void)
{
	ADMUX |= (1 << REFS0);
	
	ADCSRA |= (1 << ADEN) | (1 << ADPS0) | (1 << ADPS2);
}

void _MoveClockwise(void)
{
	SET_B_BIT(PB2);
	SET_B_BIT(PB3);
	RESET_B_BIT(PB4);
}

void _MoveCounterClockwise(void)
{
	RESET_B_BIT(PB2);
	SET_B_BIT(PB3);
	SET_B_BIT(PB4);
}

void _HaltMovement(void)
{
	RESET_B_BIT(PB2);
	RESET_B_BIT(PB3);
	RESET_B_BIT(PB4);
}

void _InitTimer1Behavior(void)
{
	OCR1A = _TIMER_OVERFLOW_TIME;
	TCCR1A = 0;
	TCCR1B = (1 << WGM12) | (1 << CS12) | (1 << CS10);
	TIMSK |= _BV(OCIE1A);
}

_Limit _IsInLimit(int left, int right)
{
	int _left = left;
	int _right = right;
	int _swapped = 0;
	
	if(left < right)
	{
		_left = right;
		_right = left;
		_swapped = 1;
	}
	
	if(LIMIT < _left-_right)
	{
		if(!_swapped)
			return VALUE_TOWARDS_LEFT;
		else
			return VALUE_TOWARDS_RIGHT;
	}
	
	return VALUE_IN_LIMIT;
}

ISR(TIMER1_COMPA_vect)
{
	int left_eye_value = _GetADCValue(CHANNEL_ACTIVE_LEFT_EYE);
	int right_eye_value = _GetADCValue(CHANNEL_ACTIVE_RIGHT_EYE);
	
	printf("%d %d\n", left_eye_value, right_eye_value);
	
	_Limit val = _IsInLimit(left_eye_value, right_eye_value);
	
	do
	{
		if(VALUE_IN_LIMIT == val)
		{
			_HaltMovement();
			break;
		}
		
		if(VALUE_TOWARDS_LEFT == val)
		{
			_MoveClockwise();
			break;
		}
		
		if(VALUE_TOWARDS_RIGHT == val)
		{
			_MoveCounterClockwise();
			break;
		}
	}while(0);
}

int main()
{
	DDRB = BDIR;
	
	_InitADCBehavior();
	_InitTimer1Behavior();
	
	sei();
		
	PORTB = 0;
	
	LOOP_ALL;

	return 0;
}
